import django
import sys
import subprocess
import time
# from subprocess import Popen
import re
from pycvesearch import CVESearch
from django.shortcuts import render, redirect, HttpResponse
from django.contrib.auth.decorators import login_required

from bs4 import BeautifulSoup

from urllib.request import Request, urlopen

from .models import DjangoVersion, InfoCVE, DjangoVersionCVE, DjangoVersionLTS
from mysite.settings import INSTALLED_APPS as INSTALLED_APPS


@login_required
def update_page(request):
    try:
        cves = DjangoVersionCVE.objects.get(django__django_version=get_current_django_version()).cve.all().order_by('cve_id').reverse()
    except DjangoVersionCVE.DoesNotExist:
        get_cves_for_version(request)
        try:
            cves = DjangoVersionCVE.objects.get(django__django_version=get_current_django_version()).cve.all().order_by('cve_id').reverse()
        except:
            cves = None

    django_version_info = DjangoVersionLTS.objects.all().order_by('base_version')
    current_django_version = get_current_django_version()
    base_django_version = current_django_version.split('.')[0] + "." + current_django_version.split('.')[1]
    current_base_version_info = DjangoVersionLTS.objects.get(base_version=base_django_version)
    recommended_version = get_recommended_version(django_version_info)
    recommended_base_version = str(recommended_version.split('.')[0] + "." + recommended_version.split('.')[1])
    ids_to_look_closer_at_changelog = get_ids_from_changelog_based_on_dot_file(recommended_version, recommended_base_version)
    changelog_info =  get_changelog_on_id(ids_to_look_closer_at_changelog, recommended_base_version)
    changelog_url = "https://docs.djangoproject.com/en/" + recommended_base_version + "/releases/" + recommended_base_version + ""

    # cves = InfoCVE.objects.filter(django_version='1.10.1').order_by('cve_id').reverse()
    # get_django_supported_and_lts_versions()
    generate_dot_and_png_file()
    return render(request, 'updating_service/info_page.html', {
        'cves': cves, 
        'django_version_info': django_version_info,
        'current_django_version': get_current_django_version(),
        'current_base_version_info': current_base_version_info,
        'ids_to_look_closer_at_changelog' : ids_to_look_closer_at_changelog, 
        'recommended_version': get_recommended_version(django_version_info),
        'recommended_base_version': recommended_base_version,
        'changelog_info': changelog_info,
        'changelog_url': changelog_url,
    })


def get_recommended_version(django_version_info):
    recommended_version = None
    for version in django_version_info:
        if version.supported_version:
            recommended_version = version.latest_version_released
            break

    return recommended_version


@login_required
def approve_update(request):
    if request.method == 'POST':
        version_number = get_recommended_version(DjangoVersionLTS.objects.all().order_by('base_version'))
        # print(version_number)
        # p = subprocess.check_output([sys.executable, '-m', 'cp', 'env', 'env-backup'])
        # print("DONE")
        # reqs = subprocess.check_output([sys.executable, '-m', 'pip', 'install', 'django=='+version_number])   

        reqs = subprocess.call([sys.executable, '-m' 'pip', 'install', 'django=='+version_number])
        # p_status = reqs.wait()
        # OUTDATED_PIP_LIST = reqs.decode('ascii').split('\r\n')[2:-1] if platform.system() == 'Windows' else reqs.decode('ascii').split('\n')[2:-1]
        # print(OUTDATED_PIP_LIST)
    return HttpResponse('')


def generate_dot_and_png_file():
    subprocess.call([sys.executable, './manage.py', 'graph_models', '-a', '-g', '-o', 'updating_service/static/img/my_project_visualized.png'])    


def get_current_django_version():
    version_number = str(django.VERSION).strip('()').strip(' ').split(', ')[:3]
    return version_number[0] + "." + version_number[1] + "." + version_number[2]


def get_ids_from_changelog_based_on_dot_file(recommended_version, recommended_base_version):
    url = "https://docs.djangoproject.com/en/" + recommended_base_version + "/releases/" + recommended_base_version + "/"

    req = Request(url, headers={'User-Agent': 'Mozilla/5.0'})

    webpage = urlopen(req)
    soup = BeautifulSoup(webpage, 'html.parser')

    ul = soup.find(role="complementary").find_all('ul', {'class': ''})

    matched_elements = searching(parse_ul(ul[0]), INSTALLED_APPS, [url], url)
    return matched_elements[1:]


def parse_ul(elem):
    result = {}
    for sub in elem.find_all('li', recursive=False):
        if sub.a is None:
            continue
        data = {k: v for k, v in sub.a.attrs.items()
                if k != 'class'}
        if sub.ul is not None:
            data['children'] = parse_ul(sub.ul)
        result[sub.a.get_text(strip=True)] = data
    return result


def searching(parsed_ul, model_graph, matched, url):
    for k1, v1 in parsed_ul.items():
        try:
            if k1 in model_graph:
                matched.append(parsed_ul[k1]['href'])
            else:
                searching(v1['children'], model_graph, matched, url)

        except KeyError:
            continue

    return matched


def get_changelog_on_id(model_attributes, recommended_base_version):
    url = "https://docs.djangoproject.com/en/" + recommended_base_version + "/releases/" + recommended_base_version + "/" 
    url2 = "https://docs.djangoproject.com/en/" + recommended_base_version + "/"

    req = Request(url, headers={'User-Agent': 'Mozilla/5.0'})

    webpage = urlopen(req)
    soup = BeautifulSoup(webpage, 'html.parser')

    returned_list = []

    for attribute in model_attributes:
        div_id = 's-' + attribute.strip("#")

        ul = soup.find('div', {'class': 'section', 'id': div_id})

        for link in ul.findAll('a', attrs={'href': re.compile("../../")}):
            link['href'] = link['href'].replace("../../", url2)
            link['target'] = "_blank"

        for element in ul.findAll('a', attrs={'title': 'Permalink to this headline'}):
            element.replaceWith("")

        full_list = ""
        for x in ul.find_all('ul'):
            full_list += str(x)
      
        returned_list.append(full_list)

    return returned_list


@login_required
def get_cves_for_version(request):
    add_base_and_supported_to_database()
    cve_list = get_all_cve_on_given_version(get_current_django_version())
    cve_element = None
    django_version_instance = None
    current_django_version = get_current_django_version()
    base_django_version = current_django_version.split('.')[0] + "." + current_django_version.split('.')[1]
    try:
        django_version_instance = DjangoVersion.objects.get(django_version=current_django_version)
    except DjangoVersion.DoesNotExist:
        django_version_instance = DjangoVersion.objects.create(
            django_version=get_current_django_version(),
            base_version_relation=DjangoVersionLTS.objects.get(base_version=base_django_version)
        )
    django_version_instance = DjangoVersion.objects.get(django_version=current_django_version)
    for i in range(len(cve_list) -1):
        if (i%2 == 0):
            try:
                cve_element = InfoCVE.objects.get(cve_id=cve_list[i])
            except InfoCVE.DoesNotExist:
                cve_element = InfoCVE.objects.create(
                    cve_id=cve_list[i],
                    comment=cve_list[i+1]
                )
            finally:
                try:
                    joined_instance = (DjangoVersionCVE.objects.get(django__django_version=current_django_version))
                except DjangoVersionCVE.DoesNotExist:
                    joined_instance = DjangoVersionCVE.objects.create(django=django_version_instance)
                    joined_instance.save()
                finally:
                    joined_instance.cve.add(cve_element)
                    joined_instance.save()

    generate_dot_and_png_file()

    return redirect('update_page')


def get_all_cve_on_given_version(VERSION_NUMBER):
    cve = CVESearch()
    returned_data = cve.search('djangoproject/django:' + VERSION_NUMBER)
    cve_list = []

    for element in returned_data:
        cve_list.append(element.get('id'))
        cve_list.append(element.get('summary'))

    return cve_list


def add_base_and_supported_to_database():
    version_list = get_django_supported_and_lts_versions()
    for element in version_list:
        try:
            django_version_lts_object = (DjangoVersionLTS.objects.get(base_version=element))
            django_version_lts_object.supported_version = (version_list[element]['supported'])
            django_version_lts_object.lts = (version_list[element]['LTS'])
            django_version_lts_object.latest_version_released = (version_list[element]['latest'])
            django_version_lts_object.mainstream = (version_list[element]['mainstream'])
            django_version_lts_object.extended = (version_list[element]['extended'])            
            django_version_lts_object.save()
        except DjangoVersionLTS.DoesNotExist:
            django_version_lts_object = DjangoVersionLTS.objects.create(
                base_version=element,
                supported_version = version_list[element]['supported'],
                lts = version_list[element]['LTS'],
                latest_version_released = version_list[element]['latest'],
                mainstream = version_list[element]['mainstream'],
                extended = version_list[element]['extended']
            )
            django_version_lts_object.save()
        # finally:
        #     joined_instance.cve.add(cve_element)                    
        #     joined_instance.save()
        # print(version_list[element])
        # {'supported': True, 'LTS': False, 'latest': '2.0.5', 'mainstream': 'August 2018', 'extended': 'April 2019'}



def get_django_supported_and_lts_versions():
    req = Request("https://www.djangoproject.com/download/", headers={'User-Agent': 'Mozilla/5.0'})
    webpage = urlopen(req)
    soup = BeautifulSoup(webpage, 'html.parser')

    table = soup.find('table', attrs={'class': 'django-supported-versions'})

    version_support = {}

    for row in table.findAll('tr', {'class': ''}):
        columns = row.find_all('td')
        counter = 0
        column_info = []
        version_number = None
        for column in columns:
            column_info = column.get_text()
            # print(column_info.split(' '))
            if(counter == 0):
                version_number = column_info.split(' ')[0]
                if len(column_info.split(' ')) >= 2 and column_info.split(' ')[1] == 'LTS':
                    version_support[version_number] = {'supported': True, 'LTS': True, 'latest': None, 'mainstream': None, 'extended': None}
                else:
                    version_support[version_number] = {'supported': True, 'LTS': False, 'latest': None, 'mainstream': None, 'extended': None}
                # break
            elif(counter == 1):
                version_support[version_number]['latest'] = column_info
            elif(counter == 2): 
                version_support[version_number]['mainstream'] = column_info
            elif(counter == 3): 
                version_support[version_number]['extended'] = column_info
            counter += 1

    for row in table.findAll('tr', {'class': 'unsupported'}):
        columns = row.find_all('td')
        counter = 0
        column_info = []
        version_number = None
        for column in columns:
            column_info = column.get_text()
            # print(column_info.split(' '))
            
            if(counter == 0):
                version_number = column_info.split(' ')[0]
                if len(column_info.split(' ')) >= 2 and column_info.split(' ')[1] == 'LTS':
                    version_support[version_number] = {'supported': False, 'LTS': True, 'latest': None, 'mainstream': None, 'extended': None}
                else:
                    version_support[version_number] = {'supported': False, 'LTS': False, 'latest': None, 'mainstream': None, 'extended': None}

            elif(counter == 1):
                    version_support[version_number]['latest'] = column_info
            elif(counter == 2): 
                version_support[version_number]['mainstream'] = column_info
            elif(counter == 3): 
                version_support[version_number]['extended'] = column_info
            counter += 1

    return version_support